#!/usr/local/bin/ruby1.9.2
require 'rubygems'
require 'ruby-debug'
require 'socket'
require 'readline'

trap('INT') { teardown }

class Socket
  def pretty_address
    remote_address.inspect_sockaddr
  end
end

def teardown
  @connections.keys.each {|address| close(address)}
end

def ask(socket)
  @connections ||= {}
  prompt = socket ? "[#{socket.pretty_address}] " : ""
  prompt << "conn:#{@connections.size} > "
  answer = Readline.readline(prompt, true)
  teardown && exit unless answer
  answer.chomp.strip
end

def use(address)
  host, port = address.split(':')
  if host.nil? || port.nil?
    puts "invalid address #{address}"
    return
  end

  @connections ||= {}
  socket = @connections[address]
  if socket
    puts "using previous connection to #{address}"
  else
    puts "establishing new connection to #{address}"
    socket = Socket.new(Socket::AF_INET, Socket::SOCK_STREAM, 0)
    sockaddr = Socket.pack_sockaddr_in(port, host)
    socket.connect(sockaddr)
    @connections[address] = socket
  end

  return socket
end

def close(address)
  socket = @connections[address]
  if socket
    puts "closing connection to #{address}"
    socket.close
    @connections[address] = nil
  else
    puts "there is no connection to #{address}. do nothing"
  end
  address
end

def list
  connections = @connections.map do |address, socket|
    "#{address} => #{socket.pretty_address}"
  end
  puts connections
end

current_socket = nil

loop do
  command = ask(current_socket)
  next if command.nil? || command.empty?
  action, address = command.split(' ')
  case action
  when 'list'
    list
  when 'use'
    current_socket = use(address)
  when 'close'
    current_address = current_socket.try(:pretty_address)
    if close(address || current_address) == current_address
      current_socket = nil
    end
  else
    if current_socket
      current_socket.puts(command)
      current_socket.flush
      length = current_socket.gets.to_i
      puts current_socket.read(length)
    else
      puts "offline. connect to server with 'use host:port'"
    end
  end
end
